#!/bin/sh
#------------------ Environment Variables ----------------------------
# workspace
WORKSPACE=$(pwd)
echo "${WORKSPACE}"

#flutter版本号，比如1.22.4
FLUTTER_VERSION="1.22.4"
# FLUTTER 目录名
FLUTTER_FOLDER="flutter-$FLUTTER_VERSION"

prepare_environment() {
  
  # 配置Flutter中国镜像
  export PUB_HOSTED_URL=https://pub.flutter-io.cn
  export FLUTTER_STORAGE_BASE_URL=https://storage.flutter-io.cn
  
  # 安装flutter
  install_flutter
  
  # 环境变量配置
  echo "Export Environment Variables"
  export FLUTTER_ROOT="${WORKSPACE}/flutter-sdk/flutter-1.22.4"
  export PATH=${FLUTTER_ROOT}/bin:${PATH}
  echo "${FLUTTER_ROOT}"
  echo "${PATH}"
  
  # 安装flutter sdk
  install_flutter_sdk

  # 签名证书
  echo "check in keychain"
  security set-key-partition-list -S apple-tool:,apple: -k 123456 "$HOME/Library/Keychains/baidumapsdk-flutter.keychain-db"
  security list-keychain -s "$HOME/Library/Keychains/baidumapsdk-flutter.keychain-db"
}

# 安装flutter
install_flutter() {
  cd ${WORKSPACE}
  # 检查是否已经安装，存在则删除重新安装，不存在则安装
  if [ -d flutter-sdk ];then
    echo 'flutter has installed!!!'
  else
   mkdir flutter-sdk
   cd flutter-sdk
   wget "https://github.com/flutter/flutter/archive/$FLUTTER_VERSION.zip"
   echo "解压zip"
   unzip $FLUTTER_VERSION.zip
   rm -rf $FLUTTER_VERSION.zip
  fi
}

# 安装flutter sdk
install_flutter_sdk() {
  cd ${WORKSPACE}
  echo "init git"
  ls
  cd flutter-sdk/${FLUTTER_FOLDER}
  git init
  git remote add origin https://github.com/flutter/flutter.git
  git remote -v
  git fetch origin master
  git checkout -f -b master origin/master
  git clean -xfd # 强制运行删除忽略不识别、未被添加git仓库的文件
  git pull
  flutter channel stable
  flutter doctor
  flutter --version
  
  cd ${WORKSPACE}
  echo "Clean products"
  flutter clean
  rm -rf ${out}
  if [[ -f "${FLUTTER_ROOT}/bin/cache/lockfile" ]] ;then
    echo "remove lockfile"
    rm -rf ${FLUTTER_ROOT}/bin/cache/lockfile
  fi
}


start_build() {
  
  echo "--------------flutter build:flutter安装依赖"
  flutter doctor -v

  echo "pub get..."
  flutter pub get
  
  cd ${WORKSPACE}
  echo "make output dir:${out}"
  mkdir ${out}
  
  # 修改AK
  cd lib/
  sed -i'.bak' 's/请在此输入您在开放平台上申请的API_KEY/wL7r0Gdc1SBfFGYK1jRsIDVESNSLrQK0/g' main.dart
  cd ..
  
  echo "开始编译"
  echo "building..."
  flutter build ios --release
  if [ -d build/ios/iphoneos/Runner.app ]
    then
      mkdir ${out}/Payload
      echo "copy Runner.app"
      cp -r build/ios/iphoneos/Runner.app ${out}/Payload
      echo "generate dSYM files"
      dsymutil build/ios/iphoneos/Runner.app/Runner Runner.dSYM
      dsymutil build/ios/iphoneos/Runner.app/Runner/Frameworks/App.framework/App App.dSYM
      cd ${out}
      echo 'zip Payload to ipa'
      zip -r -m ios-$(date "+%Y%m%d%H%M").ipa Payload
      cd ..
      echo "succeed😄"
  else
    echo "Failed😭"
  fi
}
#产物路径
out="output"
prepare_environment
start_build
